package com.sourcesense.documents;

import java.io.File;
import java.net.URI;
import java.net.URL;

import org.junit.Before;
import org.junit.Test;

import com.artofsolving.jodconverter.openoffice.connection.SocketOpenOfficeConnection;
import com.sun.star.beans.PropertyValue;
import com.sun.star.beans.XPropertySet;
import com.sun.star.container.XIndexReplace;
import com.sun.star.frame.XComponentLoader;
import com.sun.star.frame.XController;
import com.sun.star.lang.*;
import com.sun.star.lang.IllegalArgumentException;
import com.sun.star.lang.IndexOutOfBoundsException;
import com.sun.star.text.*;
import com.sun.star.uno.UnoRuntime;
import static org.junit.Assert.assertEquals;

public class TocWithHyperlink {

    private XTextDocument xTextDocument;

    @Before
    public void setup() throws Exception {
	URL resource = getClass().getResource("test-doc.rtf");
	URI uri = new URI("test-doc.rtf");
	String sUrl = "test-doc.rtf";
	XComponentLoader desktop = new SocketOpenOfficeConnection().getDesktop();
	File file = new File(sUrl);
	assertEquals("test-doc.rtf", file.getName());
	// assertEquals("test-doc.rtf", file.getAbsolutePath());
	XComponent xComp = desktop.loadComponentFromURL("file://"
		+ file.getAbsolutePath(), "_blank", 0, (new PropertyValue[0]));
	xTextDocument = ((XTextDocument) UnoRuntime.queryInterface(
		com.sun.star.text.XTextDocument.class, xComp));
    }

    @Test
    public void shouldCreateTocWithHyperlinks() throws Exception {
	XPropertySet indexPropertySet = contentIndex();

	indexPropertySet
		.setPropertyValue(DocumentIndex.CREATE_FROM_OUTLINE, Boolean.TRUE);
	indexPropertySet.setPropertyValue(
		DocumentIndex.CREATE_FROM_LEVEL_PARAGRAPH_STYLES, Boolean.TRUE);
	indexPropertySet.setPropertyValue("IsProtected", Boolean.FALSE);
	indexPropertySet.setPropertyValue(DocumentIndex.LEVEL, Short.parseShort("10"));

	XIndexReplace levelParagraphStyle = (XIndexReplace) UnoRuntime.queryInterface(
		XIndexReplace.class, indexPropertySet
			.getPropertyValue("LevelParagraphStyles"));
	String[] props0 = stringArray("TITOLO1");
	String[] props1 = stringArray("TITOLO2");
	levelParagraphStyle.replaceByIndex(0, props0);
	levelParagraphStyle.replaceByIndex(1, props1);

	XIndexReplace levelFormat = (XIndexReplace) UnoRuntime.queryInterface(
		XIndexReplace.class, indexPropertySet.getPropertyValue("LevelFormat"));
	addHyperlinkIndex(levelFormat);

	XTextViewCursor viewCursor = viewCursor();
	XPageCursor xPageCursor = (XPageCursor) UnoRuntime.queryInterface(
		XPageCursor.class, viewCursor);
	xPageCursor.jumpToPage(Short.parseShort("2"));

	XTextContent toc = ((XTextContent) UnoRuntime.queryInterface(XTextContent.class,
		indexPropertySet));
	xTextDocument.getText().insertTextContent(viewCursor, toc, false);
	XDocumentIndex xDocIndex = (XDocumentIndex) UnoRuntime.queryInterface(
		XDocumentIndex.class, indexPropertySet);
	xDocIndex.update();

    }

    private void addHyperlinkIndex(XIndexReplace levelFormat)
	    throws IndexOutOfBoundsException, WrappedTargetException,
	    IllegalArgumentException {
	for (int i = 0; i < 2; i++) {
	    PropertyValue[][] propertyValuesSequence = new PropertyValue[6][];

	    PropertyValue[] propertyValuesEntry0 = new PropertyValue[2];
	    propertyValuesEntry0[0] = OOUtils
		    .property("TokenType", "TokenHyperlinkStart");
	    propertyValuesEntry0[1] = OOUtils.property("CharacterStyleName",
		    "Internet Link");
	    propertyValuesSequence[0] = propertyValuesEntry0;

	    PropertyValue[] propertyValuesEntry1 = new PropertyValue[2];
	    propertyValuesEntry1[0] = OOUtils.property("TokenType", "TokenEntryNumber");
	    propertyValuesEntry1[1] = OOUtils.property("CharacterStyleName", "");
	    propertyValuesSequence[1] = propertyValuesEntry1;

	    PropertyValue[] propertyValuesEntry2 = new PropertyValue[2];
	    propertyValuesEntry2[0] = OOUtils.property("TokenType", "TokenEntryText");
	    propertyValuesEntry2[1] = OOUtils.property("CharacterStyleName", "");
	    propertyValuesSequence[2] = propertyValuesEntry2;

	    PropertyValue[] propertyValuesEntry3 = new PropertyValue[2];
	    propertyValuesEntry3[0] = OOUtils.property("TokenType", "TokenHyperlinkEnd");
	    propertyValuesEntry3[1] = OOUtils.property("CharacterStyleName",
		    "Internet Link");
	    propertyValuesSequence[3] = propertyValuesEntry3;

	    PropertyValue[] propertyValuesEntry4 = new PropertyValue[4];
	    propertyValuesEntry4[0] = OOUtils.property("TokenType", "TokenTabStop");
	    propertyValuesEntry4[1] = OOUtils.property("TabStopRightAligned",
		    Boolean.TRUE);
	    propertyValuesEntry4[2] = OOUtils.property("TabStopFillCharacter", ".");
	    propertyValuesEntry4[3] = OOUtils.property("CharacterStyleName", "");
	    propertyValuesSequence[4] = propertyValuesEntry4;

	    PropertyValue[] propertyValuesEntry5 = new PropertyValue[2];
	    propertyValuesEntry5[0] = OOUtils.property("TokenType", "TokenPageNumber");
	    propertyValuesEntry5[1] = OOUtils.property("CharacterStyleName", "");
	    propertyValuesSequence[5] = propertyValuesEntry5;

	    levelFormat.replaceByIndex(i, propertyValuesSequence);

	}
    }

    private XPropertySet contentIndex() throws Exception {
	XMultiServiceFactory mxDocFactory = (XMultiServiceFactory) UnoRuntime
		.queryInterface(XMultiServiceFactory.class, xTextDocument);
	XPropertySet indexPropertySet = (XPropertySet) UnoRuntime.queryInterface(
		XPropertySet.class, mxDocFactory
			.createInstance("com.sun.star.text.ContentIndex"));
	return indexPropertySet;
    }

    private XTextViewCursor viewCursor() {
	XController currentController = xTextDocument.getCurrentController();
	XTextViewCursorSupplier xViewCursorSupplier = (XTextViewCursorSupplier) UnoRuntime
		.queryInterface(XTextViewCursorSupplier.class, currentController);
	XTextViewCursor viewCursor = xViewCursorSupplier.getViewCursor();
	return viewCursor;
    }

    private String[] stringArray(String style) {
	String[] props = new String[1];
	props[0] = style;
	return props;
    }

}
